// Tut07 CPSC 453 Created By: Andrew Owens
// This is a (very) basic program to
// 1) load shaders from external files, and make a shader program
// 2) make Vertex Array Object and Vertex Buffer Object for the triangle

// take a look at the following sites for further readings:
// opengl-tutorial.org -> The first triangle
// antongerdelan.net -> shaders pipeline explained
// ogldev.atspace.co.uk -> good resource 


#include<iostream>

#include<GL/glew.h> // include BEFORE glut
#include<GL/glut.h>

#include "ShaderTools.h"

using std::cout;
using std::endl;
using std::cerr;

GLuint vaoID;
GLuint basicProgramID;
GLuint vertBufferID;

// function declarations
void display();
void init();
void generateIDs();
void setupVAO();
void loadBuffer();
int main( int, char** );
// function declarations

void display()
{
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glUseProgram( basicProgramID );
	glBindVertexArray( vaoID );
	glDrawArrays( GL_TRIANGLES, 0, 3 );

	glutSwapBuffers();
}

void generateIDs()
{
	
	std::string vsSource = loadShaderStringfromFile( "./basic_vs.glsl" );
	std::string fsSource = loadShaderStringfromFile( "./basic_fs.glsl" );
	basicProgramID = CreateShaderProgram( vsSource, fsSource );

	// load IDs given from OpenGL
	glGenVertexArrays( 1, &vaoID );
	glGenBuffers( 1, &vertBufferID );
}

void setupVAO()
{
	glBindVertexArray( vaoID );
	glEnableVertexAttribArray( 0 ); // match layout # in shader
	
	glBindBuffer( GL_ARRAY_BUFFER, vertBufferID );
	glVertexAttribPointer(
		0,		// attribute layout # above
		3,		// # of components (ie XYZ )
		GL_FLOAT,	// type of components
		GL_FALSE,	// need to be normalized?
		0,		// stride
		(void*)0	// array buffer offset
	);

	glBindVertexArray( 0 ); // reset to default		
}

void loadBuffer()
{
	// Just basic layout of floats, for a triagle
	// 3 floats per vertex, 3 vertices
	static const float verts[] = {
		-1.0f,	-1.0f, 	0.0f,
		1.0f,	-1.0f, 	0.0f,
		0.0f,	1.0f,	0.0f
	};

	glBindBuffer( GL_ARRAY_BUFFER, vertBufferID );
	glBufferData(	GL_ARRAY_BUFFER,
			sizeof(float)*9,
			verts,
			GL_STATIC_DRAW );
}

void init()
{
	glEnable( GL_DEPTH_TEST );

	// SETUP SHADERS, BUFFERS, VAOs

	generateIDs();
	setupVAO();
	loadBuffer();
}

int main( int argc, char** argv )
{
	glutInit( &argc, argv );
	// Setup FB configuration
	glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH );
	
	glutInitWindowSize( 800, 600 );
	glutInitWindowPosition( 0, 0 );

	glutCreateWindow( "Tut07" );

	glewExperimental=true; // Needed in Core Profile
	// Comment out if you want to us glBeign etc...
	if( glewInit() != GLEW_OK )
	{
		cerr << "Failed to initialize GLEW" << endl;
		return -1;
	}
	cout << "GL Version: :" << glGetString(GL_VERSION) << endl;

	glutDisplayFunc( display );

	init(); // our own initialize stuff func

	glutMainLoop();
	return 0;
}
